/*
 * Copyright (c) 2011-2012 Gony
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE ABOVE LISTED COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package jp.gonypage.dancingmissiles;

import javax.microedition.khronos.opengles.GL11;

import android.content.Context;
import android.content.res.Resources;

public class DancingMissilesScene implements Scene {
	public DancingMissilesScene(Context context) {
		Settings settings = Settings.getInstance();
		Resources resources = context.getResources();
		smokeTexture = new Texture(resources, R.raw.smoke);
		explosionTexture = new Texture(resources, R.raw.explosion);
		if (settings.getBackgroundType() == Settings.BACKGROUND_SEA) {
			skyTexture = new Texture(resources, R.raw.seasky);
			groundTexture = new Texture(resources, R.raw.sea);
		}
		else if (settings.getBackgroundType() == Settings.BACKGROUND_FOREST) {
			skyTexture = new Texture(resources, R.raw.forestsky);
			groundTexture = new Texture(resources, R.raw.forest);
		}
		else {
			skyTexture = new Texture(resources, R.raw.desertsky);
			groundTexture = new Texture(resources, R.raw.desert);
		}
		missileTexture = new Texture(resources, R.raw.missile);
		font = new Font(resources);

		int missileNum = settings.getMissileNum();
		int smokeInterval = settings.getSmokeInterval();
		missiles = new ActorList(missileNum);
		for (int i = 0;i < missileNum;i++) {
			missiles.add(new Missile(this, smokeInterval));
		}
		smokes = new SmokeList(missileNum * SMOKE_NUM, this);
		explosions = new ExplosionList(EXPLOSION_NUM, this);
		target = new Target();
		background = new Background(this);
		camera = new Camera(target);
		speedLines = new SpeedLines(this);
	}

	public void onPause() {
		// TODO Auto-generated method stub

	}

	public void onResume() {
		// TODO Auto-generated method stub

	}

	public void invalidateTextues(GL11 gl) {
		smokeTexture.invalidate(gl);
		explosionTexture.invalidate(gl);
		skyTexture.invalidate(gl);
		groundTexture.invalidate(gl);
	}

	public void process() {
		Settings settings = Settings.getInstance();
		int missileNum = settings.getMissileNum();
		int smokeInterval = settings.getSmokeInterval();
		if (missiles.getNum() < missileNum) {
			missiles.add(new Missile(this, smokeInterval));
		}
		target.process();
		missiles.process();
		smokes.process();
		explosions.process();
		camera.process();
		background.process();
		speedLines.process();
	}

	public void draw(GL11 gl, float viewDifference) {
		float ratioX;
		float ratioY;
		if (surfaceWidth < surfaceHeight) {
			ratioX = 0.7f;
			ratioY = ratioX / (float)surfaceWidth * (float)surfaceHeight;
		}
		else {
			ratioY = 0.7f;
			ratioX = ratioY / (float)surfaceHeight * (float)surfaceWidth;
		}

		initializeRenderingParameters(gl);
		gl.glClear(GL11.GL_COLOR_BUFFER_BIT | GL11.GL_DEPTH_BUFFER_BIT);

		gl.glMatrixMode(GL11.GL_PROJECTION);
		gl.glLoadIdentity();
		gl.glFrustumf(-ratioX, ratioX, -ratioY, ratioY, 1.0f, 20000.0f);
		gl.glMatrixMode(GL11.GL_MODELVIEW);
		gl.glLoadIdentity();
		camera.draw(gl);

		gl.glEnable(GL11.GL_DEPTH_TEST);
		gl.glEnable(GL11.GL_FOG);
		gl.glFogx(GL11.GL_FOG_MODE, GL11.GL_LINEAR);
		background.draw(gl);

		missiles.draw(gl);
		Settings settings = Settings.getInstance();
		if (settings.getTargetShown()) {
			target.draw(gl);
		}
		smokes.draw(gl);
		explosions.draw(gl);
		speedLines.draw(gl);

		gl.glDisable(GL11.GL_FOG);
		gl.glDisable(GL11.GL_DEPTH_TEST);

		if (settings.getInformationShown()) {
			gl.glMatrixMode(GL11.GL_PROJECTION);
			gl.glLoadIdentity();
			gl.glOrthof(0.0f, surfaceWidth, surfaceHeight, 0.0f, 10.0f, -10.0f);
			gl.glMatrixMode(GL11.GL_MODELVIEW);
			gl.glLoadIdentity();
			gl.glTranslatef(0.0f, 64.0f, 0.0f);
			gl.glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
			font.draw(gl, String.format("FRAME RATE: %2.2f (FPS)", fps));
		}

		gl.glFlush();
	}

	public void setSurfaceSize(int width, int height) {
		surfaceWidth = width;
		surfaceHeight = height;
	}

	public void setTouches(Touches aTouches) {
		// TODO Auto-generated method stub
	}

	public void setFps(float aFps) {
		fps = aFps;
	}

	public Texture getSmokeTexture() {
		return smokeTexture;
	}

	public Texture getExplosionTexture() {
		return explosionTexture;
	}

	public Texture getSkyTexture() {
		return skyTexture;
	}

	public Texture getGroundTexture() {
		return groundTexture;
	}

	public Texture getMissileTexture() {
		return missileTexture;
	}

	public ActorList getMissiles() {
		return missiles;
	}
	
	public ActorList getSmokes() {
		return smokes;
	}

	public ActorList getExplosions() {
		return explosions;
	}

	public Camera getCamera() {
		return camera;
	}
	
	public Target getTarget() {
		return target;
	}

	private void initializeRenderingParameters(GL11 gl) {
		int backgroundType = Settings.getInstance().getBackgroundType();

		if (backgroundType == Settings.BACKGROUND_SEA) {
			gl.glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
		}
		else if (backgroundType == Settings.BACKGROUND_FOREST) {
			gl.glClearColor(0.0f, 0.1f, 0.5f, 1.0f);
		}
		else {
			gl.glClearColor(0.5f, 0.3f, 0.0f, 1.0f);
		}

		if (backgroundType == Settings.BACKGROUND_SEA) {
			float[] fogColor = {
					0.0f, 0.0f, 0.0f, 1.0f
			};

			gl.glFogf(GL11.GL_FOG_START, 1000.0f);
			gl.glFogf(GL11.GL_FOG_END, 6000.0f);
			gl.glFogfv(GL11.GL_FOG_COLOR, fogColor, 0);
		}
		else if (backgroundType == Settings.BACKGROUND_FOREST) {
			float[] fogColor = {
					0.0f, 0.1f, 0.5f, 1.0f
			};

			gl.glFogf(GL11.GL_FOG_START, 3000.0f);
			gl.glFogf(GL11.GL_FOG_END, 6000.0f);
			gl.glFogfv(GL11.GL_FOG_COLOR, fogColor, 0);
		}
		else {
			float[] fogColor = {
					0.5f, 0.3f, 0.0f, 1.0f
			};

			gl.glFogf(GL11.GL_FOG_START, 1000.0f);
			gl.glFogf(GL11.GL_FOG_END, 6000.0f);
			gl.glFogfv(GL11.GL_FOG_COLOR, fogColor, 0);
		}

		if (backgroundType == Settings.BACKGROUND_SEA) {
			float[] lightPosition = {
					1.0f, 1.0f, 0.0f, 0.0f
			};
			float[] ambient = {
					0.8f, 0.8f, 0.8f, 0.0f
			};
			float[] diffuse = {
					1.0f, 1.0f, 1.0f, 0.0f
			};
			float[] specular = {
					0.0f, 0.0f, 0.0f, 0.0f
			};
			gl.glLightfv(GL11.GL_LIGHT0, GL11.GL_POSITION, lightPosition, 0);
			gl.glLightfv(GL11.GL_LIGHT0, GL11.GL_AMBIENT, ambient, 0);
			gl.glLightfv(GL11.GL_LIGHT0, GL11.GL_DIFFUSE, diffuse, 0);
			gl.glLightfv(GL11.GL_LIGHT0, GL11.GL_SPECULAR, specular, 0);
		}
		else {
			float[] lightPosition = {
					1.0f, 1.0f, 0.0f, 0.0f
			};
			float[] ambient = {
					1.0f, 1.0f, 1.0f, 0.0f
			};
			float[] diffuse = {
					1.0f, 1.0f, 1.0f, 0.0f
			};
			float[] specular = {
					0.0f, 0.0f, 0.0f, 0.0f
			};
			gl.glLightfv(GL11.GL_LIGHT0, GL11.GL_POSITION, lightPosition, 0);
			gl.glLightfv(GL11.GL_LIGHT0, GL11.GL_AMBIENT, ambient, 0);
			gl.glLightfv(GL11.GL_LIGHT0, GL11.GL_DIFFUSE, diffuse, 0);
			gl.glLightfv(GL11.GL_LIGHT0, GL11.GL_SPECULAR, specular, 0);
		}
	}

	private final static int SMOKE_NUM = 64;
	private final static int EXPLOSION_NUM = 64;

	private Texture smokeTexture;
	private Texture explosionTexture;
	private Texture skyTexture;
	private Texture groundTexture;
	private Texture missileTexture;
	private Font font;
	private ActorList missiles;
	private ActorList smokes;
	private ActorList explosions;
	private Camera camera;
	private Target target;
	private Background background;
	private SpeedLines speedLines;
	private int surfaceWidth;
	private int surfaceHeight;
	private float fps;
}
